package kaka.cakelight;

import java.util.Arrays;

public class LedFrame {
    private Configuration config;
    private byte[] bytes;
    private int start, stride;
    private int roff, goff, boff; // RGB offsets

    public static LedFrame from(Configuration config) {
        LedFrame frame = new LedFrame();
        frame.config = config;
        switch (config.leds.type) {
            /*
             * The WS2801 strip takes its input as a plain list of 24-bit colors in RBG order (at least mine did).
             */
            case WS2801:
		frame.start = 0;
                frame.stride = 3;
                frame.roff = 0;
                frame.goff = 2;
                frame.boff = 1;
                frame.bytes = new byte[config.leds.getCount() * frame.stride];
                break;

            /*
             * The APA102 strip takes its input as:
             * <ol>
             * <li>a start frame of 4 bytes (all zeroes)</li>
             * <li>a frame of 4 bytes for each LED (111 (3 bits) + global illumination (5 bits) + BGR)</li>
             * <li>an (optional) end frame of 4 bytes (all ones)</li>
             * </ol>
             */
            case APA102:
		frame.start = 4;
                frame.stride = 4;
                frame.roff = 3;
                frame.goff = 2;
                frame.boff = 1;
                frame.bytes = new byte[4 + config.leds.getCount() * frame.stride + 4 * 2]; // 1 end frame doesn't seem to work, so we add another
                Arrays.fill(frame.bytes, 4, frame.bytes.length - 9, (byte)(0b11100000 | config.leds.brightness)); // Initiate the first byte of each LED
                Arrays.fill(frame.bytes, frame.bytes.length - 9, frame.bytes.length, (byte)0xff); // Initiate the end frame(s) with ones
                break;
        }
        return frame;
    }

    public LedFrame fillColor(int r, int g, int b) {
        return fillColor(Color.rgb(r, g, b));
    }

    public LedFrame fillColor(Color color) {
        byte r = (byte)color.r(), g = (byte)color.g(), b = (byte)color.b(); // Gamma corrected values
        for (int i = start, max = start + config.leds.getCount() * stride; i < max; i += stride) {
            bytes[i + roff] = r;
            bytes[i + goff] = g;
            bytes[i + boff] = b;
        }
	return this;
    }

    public void setLedColor(int led, Color color) {
        int offset = start + led * stride;
        bytes[offset + roff] = (byte)color.r();
        bytes[offset + goff] = (byte)color.g();
        bytes[offset + boff] = (byte)color.b();
    }

    public Color getLedColor(int led) {
        int offset = start + led * stride;
        return Color.rgb(
                bytes[offset + roff] & 0xff,
                bytes[offset + goff] & 0xff,
                bytes[offset + boff] & 0xff
        );
    }

    public byte[] getBytes() {
        return bytes;
    }

    // TODO this needs to be improved
    /** The x position of the led from 0.0-1.0. */
    public double xOf(int led) {
        /* left   */ if (led >= config.leds.cols * 2 + config.leds.rows) return 0;
        /* top    */ if (led >= config.leds.cols + config.leds.rows) return 1 - (double)(led - config.leds.cols - config.leds.rows) / config.leds.cols;
        /* right  */ if (led >= config.leds.cols) return 1;
        /* bottom */ return (double)led / config.leds.cols;
    }

    /** The y position of the led from 0.0-1.0. */
    public double yOf(int led) {
        /* left   */ if (led >= config.leds.cols * 2 + config.leds.rows) return (double)(led - config.leds.cols * 2 - config.leds.rows) / config.leds.rows;
        /* top    */ if (led >= config.leds.cols + config.leds.rows) return 0;
        /* right  */ if (led >= config.leds.cols) return 1 - (double)(led - config.leds.cols) / config.leds.rows;
        /* bottom */ return 1;
    }
}
